/* hex2dec.c */

/* (c) 2002 Josh Harding
 *
 * v1.0 - 2002.03.03
 *
 * License: This program is free for any use in any way.
 *
 */
 
#include <stdio.h>
#include <unistd.h>

#define LINELEN 150

long hex2dec(char *, int);
void print(char *, int, FILE *);
void printip(char *, FILE *);
void doio(FILE *);

main(int argc, char *argv[]) {
	int digits=0;
	FILE *outstream;

	if(argc == 1) {
		/* no filename, just run once to stdout */
		doio(stdout);
	} else {
		while(1) {
			outstream=fopen(argv[1], "w");
			if (!outstream) {
				exit(1);
			}
			doio(outstream);
			fclose(outstream);
			/* Without the usleep(1), it loops several times */
			usleep(1);
		}
	}
}

	
void doio(FILE *output) {
	char line[LINELEN];
	FILE *fp;
	if(!(fp=fopen("/proc/net/tcp", "r"))) {
		exit(1);
	}
	
	/* throw out column headers */
	fgets(line, LINELEN, fp);
	fprintf(output, "  sl  local_address   port# remote_addr     port# st   tx_queue   rx_queue\n");

	while(!feof(fp)) {
		fgets(line, LINELEN, fp);
		/* line number */
		print(line, 6, output);

		/* local IP */
		printip(&line[6], output);

		/* local port */
		fprintf(output, ":%05d ", (int) hex2dec(&line[15], 4));

		/* remote IP */
		printip(&line[20], output);

		/* remote port */
		fprintf(output, ":%05d ", (int) hex2dec(&line[29], 4));

		/* st */
		fprintf(output, "%02d ", (int) hex2dec(&line[34], 2));

		/* tx_queue */
		fprintf(output, "%10ld:", hex2dec(&line[37], 8));

		/* rx_queue */
		fprintf(output, "%10ld\n", hex2dec(&line[46], 8));
	}
	fclose(fp);
}

long hex2dec(char *hexstr, int digits) {
	int x;
	long value=0;
	for (x=0; x < digits; x++) {
		value <<= 4;
		if (hexstr[x] >= 'A' && hexstr[x] <= 'F') {
			value += (hexstr[x] - 'A' + 10);
		} else {
			value += (hexstr[x] - '0');
		}
	}
	return value;
}

void print(char *printstr, int len, FILE *output) {
	char temp[LINELEN];
	strncpy(temp, printstr, len);
	temp[len]=0;
	fprintf(output, "%s", temp);
}

void printip(char *printstr, FILE *output) {
	int a,b,c,d;
	a = hex2dec(printstr, 2);
	b = hex2dec(&printstr[2], 2);
	c = hex2dec(&printstr[4], 2);
	d = hex2dec(&printstr[6], 2);
	fprintf(output, "%03d.%03d.%03d.%03d",d,c,b,a);
}
